@@ -1,5 +1,3 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim:set ts=2 sw=2 sts=2 et: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -48,6 +46,7 @@ const { isBrowser } = require("./window/utils");
 const { setTimeout } = require("./timers");
 const unload = require("./system/unload");
 const { uuid } = require("./util/uuid");
+const { getNodeView } = require("./view/core");
 
 // Data types definition
 const valid = {
@@ -362,7 +361,6 @@ const Widget = function Widget(options) {
 exports.Widget = Widget;
 
 
-
 /**
  * WidgetView is an instance of a widget for a specific window.
  *
@@ -432,7 +430,10 @@ const WidgetViewTrait = LightTrait.compose(EventEmitterTrait, LightTrait({
     // Special case for click events: if the widget doesn't have a click
     // handler, but it does have a panel, display the panel.
     if ("click" == type && !this._listeners("click").length && this.panel)
-      this.panel.show(domNode);
+      // This kind of ugly workaround, instead we should implement
+      // `getNodeView` for the `Widget` class itself, but that's kind of
+      // hard without cleaning things up.
+      this.panel.show(null, getNodeView.implement({}, function() domNode));
   },
 
   _isInWindow: function WidgetView__isInWindow(window) {
@@ -472,6 +473,7 @@ const WidgetViewTrait = LightTrait.compose(EventEmitterTrait, LightTrait({
 
 }));
 
+
 const WidgetView = function WidgetView(baseWidget) {
   let w = WidgetViewTrait.create(WidgetView.prototype);
   w._initWidgetView(baseWidget);
@@ -479,7 +481,6 @@ const WidgetView = function WidgetView(baseWidget) {
 }
 
 
-
 /**
  * Keeps track of all browser windows.
  * Exposes methods for adding/removing widgets
@@ -601,6 +602,42 @@ BrowserWindow.prototype = {
     let palette = toolbox.palette;
     palette.appendChild(node);
 
+    if (this.window.CustomizableUI) {
+      let placement = this.window.CustomizableUI.getPlacementOfWidget(node.id);
+      if (!placement) {
+        placement = {area: 'nav-bar', position: undefined};
+      }
+      this.window.CustomizableUI.addWidgetToArea(node.id, placement.area, placement.position);
+
+      // Depending on when this gets called, we might be in the right place now. In that case,
+      // don't run the following code.
+      if (node.parentNode != palette) {
+        return;
+      }
+      // Otherwise, insert:
+      let container = this.doc.getElementById(placement.area);
+      if (container.customizationTarget) {
+        container = container.customizationTarget;
+      }
+
+      if (placement.position !== undefined) {
+        // Find a position:
+        let items = this.window.CustomizableUI.getWidgetIdsInArea(placement.area);
+        let itemIndex = placement.position;
+        for (let l = items.length; itemIndex < l; itemIndex++) {
+          let realItems = container.getElementsByAttribute("id", items[itemIndex]);
+          if (realItems[0]) {
+            container.insertBefore(node, realItems[0]);
+            break;
+          }
+        }
+      }
+      if (node.parentNode != container) {
+        container.appendChild(node);
+      }
+      return;
+    }
+
     // Search for widget toolbar by reading toolbar's currentset attribute
     let container = null;
     let toolbars = this.doc.getElementsByTagName("toolbar");
