@@ -11,7 +11,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 //// nsRandomGenerator
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsRandomGenerator, nsIRandomGenerator)
+NS_IMPL_ISUPPORTS1(nsRandomGenerator, nsIRandomGenerator)
 
 ////////////////////////////////////////////////////////////////////////////////
 //// nsIRandomGenerator
@@ -23,6 +23,7 @@ nsRandomGenerator::GenerateRandomBytes(uint32_t aLength,
                                        uint8_t **aBuffer)
 {
   NS_ENSURE_ARG_POINTER(aBuffer);
+  *aBuffer = nullptr;
 
   uint8_t *buf = reinterpret_cast<uint8_t *>(NS_Alloc(aLength));
   if (!buf)
