@@ -7,7 +7,7 @@
 
 #include "base/basictypes.h"
 
-#include "BasicLayers.h"
+#include "ClientLayerManager.h"
 #include "gfxPlatform.h"
 #if defined(MOZ_ENABLE_D3D10_LAYER)
 # include "LayerManagerD3D10.h"
@@ -15,7 +15,7 @@
 #include "mozilla/dom/TabChild.h"
 #include "mozilla/Hal.h"
 #include "mozilla/layers/CompositorChild.h"
-#include "mozilla/layers/PLayersChild.h"
+#include "mozilla/layers/PLayerTransactionChild.h"
 #include "PuppetWidget.h"
 #include "nsIWidgetListener.h"
 
@@ -36,7 +36,7 @@ InvalidateRegion(nsIWidget* aWidget, const nsIntRegion& aRegion)
 /*static*/ already_AddRefed<nsIWidget>
 nsIWidget::CreatePuppetWidget(TabChild* aTabChild)
 {
-  NS_ABORT_IF_FALSE(nsIWidget::UsePuppetWidgets(),
+  NS_ABORT_IF_FALSE(!aTabChild || nsIWidget::UsePuppetWidgets(),
                     "PuppetWidgets not allowed in this configuration");
 
   nsCOMPtr<nsIWidget> widget = new PuppetWidget(aTabChild);
@@ -74,6 +74,7 @@ NS_IMPL_ISUPPORTS_INHERITED1(PuppetWidget, nsBaseWidget,
 PuppetWidget::PuppetWidget(TabChild* aTabChild)
   : mTabChild(aTabChild)
   , mDPI(-1)
+  , mDefaultScale(-1)
 {
   MOZ_COUNT_CTOR(PuppetWidget);
 }
@@ -120,6 +121,7 @@ PuppetWidget::Create(nsIWidget        *aParent,
 void
 PuppetWidget::InitIMEState()
 {
+  MOZ_ASSERT(mTabChild);
   if (mNeedIMEStateInit) {
     uint32_t chromeSeqno;
     mTabChild->SendNotifyIMEFocus(false, &mIMEPreference, &chromeSeqno);
@@ -302,7 +304,7 @@ PuppetWidget::DispatchEvent(nsGUIEvent* event, nsEventStatus& aStatus)
 }
 
 LayerManager*
-PuppetWidget::GetLayerManager(PLayersChild* aShadowManager,
+PuppetWidget::GetLayerManager(PLayerTransactionChild* aShadowManager,
                               LayersBackend aBackendHint,
                               LayerManagerPersistence aPersistence,
                               bool* aAllowRetaining)
@@ -320,7 +322,7 @@ PuppetWidget::GetLayerManager(PLayersChild* aShadowManager,
     }
 #endif
     if (!mLayerManager) {
-      mLayerManager = new BasicShadowLayerManager(this);
+      mLayerManager = new ClientLayerManager(this);
       mLayerManager->AsShadowForwarder()->SetShadowManager(aShadowManager);
     }
   }
@@ -367,15 +369,23 @@ PuppetWidget::IMEEndComposition(bool aCancel)
 }
 
 NS_IMETHODIMP
-PuppetWidget::ResetInputState()
+PuppetWidget::NotifyIME(NotificationToIME aNotification)
 {
-  return IMEEndComposition(false);
-}
-
-NS_IMETHODIMP
-PuppetWidget::CancelComposition()
-{
-  return IMEEndComposition(true);
+  switch (aNotification) {
+    case NOTIFY_IME_OF_CURSOR_POS_CHANGED:
+    case REQUEST_TO_COMMIT_COMPOSITION:
+      return IMEEndComposition(false);
+    case REQUEST_TO_CANCEL_COMPOSITION:
+      return IMEEndComposition(true);
+    case NOTIFY_IME_OF_FOCUS:
+      return NotifyIMEOfFocusChange(true);
+    case NOTIFY_IME_OF_BLUR:
+      return NotifyIMEOfFocusChange(false);
+    case NOTIFY_IME_OF_SELECTION_CHANGE:
+      return NotifyIMEOfSelectionChange();
+    default:
+      return NS_ERROR_NOT_IMPLEMENTED;
+  }
 }
 
 NS_IMETHODIMP_(void)
@@ -418,8 +428,8 @@ PuppetWidget::GetInputContext()
   return context;
 }
 
-NS_IMETHODIMP
-PuppetWidget::OnIMEFocusChange(bool aFocus)
+nsresult
+PuppetWidget::NotifyIMEOfFocusChange(bool aFocus)
 {
 #ifndef MOZ_CROSS_PROCESS_IME
   return NS_OK;
@@ -440,19 +450,21 @@ PuppetWidget::OnIMEFocusChange(bool aFocus)
       mTabChild->SendNotifyIMETextHint(queryEvent.mReply.mString);
     }
   } else {
-    // ResetInputState might not have been called yet
-    ResetInputState();
+    // Might not have been committed composition yet
+    IMEEndComposition(false);
   }
 
   uint32_t chromeSeqno;
-  mIMEPreference.mWantUpdates = false;
+  mIMEPreference.mWantUpdates = nsIMEUpdatePreference::NOTIFY_NOTHING;
   mIMEPreference.mWantHints = false;
   if (!mTabChild->SendNotifyIMEFocus(aFocus, &mIMEPreference, &chromeSeqno))
     return NS_ERROR_FAILURE;
 
   if (aFocus) {
-    if (mIMEPreference.mWantUpdates && mIMEPreference.mWantHints) {
-      OnIMESelectionChange(); // Update selection
+    if ((mIMEPreference.mWantUpdates &
+           nsIMEUpdatePreference::NOTIFY_SELECTION_CHANGE) &&
+        mIMEPreference.mWantHints) {
+      NotifyIMEOfSelectionChange(); // Update selection
     }
   } else {
     mIMELastBlurSeqno = chromeSeqno;
@@ -467,7 +479,9 @@ PuppetWidget::GetIMEUpdatePreference()
 }
 
 NS_IMETHODIMP
-PuppetWidget::OnIMETextChange(uint32_t aStart, uint32_t aEnd, uint32_t aNewEnd)
+PuppetWidget::NotifyIMEOfTextChange(uint32_t aStart,
+                                    uint32_t aEnd,
+                                    uint32_t aNewEnd)
 {
 #ifndef MOZ_CROSS_PROCESS_IME
   return NS_OK;
@@ -487,14 +501,14 @@ PuppetWidget::OnIMETextChange(uint32_t aStart, uint32_t aEnd, uint32_t aNewEnd)
       mTabChild->SendNotifyIMETextHint(queryEvent.mReply.mString);
     }
   }
-  if (mIMEPreference.mWantUpdates) {
+  if (mIMEPreference.mWantUpdates & nsIMEUpdatePreference::NOTIFY_TEXT_CHANGE) {
     mTabChild->SendNotifyIMETextChange(aStart, aEnd, aNewEnd);
   }
   return NS_OK;
 }
 
-NS_IMETHODIMP
-PuppetWidget::OnIMESelectionChange(void)
+nsresult
+PuppetWidget::NotifyIMEOfSelectionChange()
 {
 #ifndef MOZ_CROSS_PROCESS_IME
   return NS_OK;
@@ -503,7 +517,8 @@ PuppetWidget::OnIMESelectionChange(void)
   if (!mTabChild)
     return NS_ERROR_FAILURE;
 
-  if (mIMEPreference.mWantUpdates) {
+  if (mIMEPreference.mWantUpdates &
+        nsIMEUpdatePreference::NOTIFY_SELECTION_CHANGE) {
     nsEventStatus status;
     nsQueryContentEvent queryEvent(true, NS_QUERY_SELECTED_TEXT, this);
     InitEvent(queryEvent, nullptr);
@@ -525,8 +540,7 @@ PuppetWidget::SetCursor(nsCursor aCursor)
     return NS_OK;
   }
 
-  if (!mTabChild ||
-      !mTabChild->SendSetCursor(aCursor)) {
+  if (mTabChild && !mTabChild->SendSetCursor(aCursor)) {
     return NS_ERROR_FAILURE;
   }
 
@@ -558,15 +572,22 @@ PuppetWidget::Paint()
 #endif
 
     if (mozilla::layers::LAYERS_D3D10 == mLayerManager->GetBackendType()) {
-      mAttachedWidgetListener->PaintWindow(this, region, 0);
+      mAttachedWidgetListener->PaintWindow(this, region);
+    } else if (mozilla::layers::LAYERS_CLIENT == mLayerManager->GetBackendType()) {
+      // Do nothing, the compositor will handle drawing
+      if (mTabChild) {
+        mTabChild->NotifyPainted();
+      }
     } else {
       nsRefPtr<gfxContext> ctx = new gfxContext(mSurface);
       ctx->Rectangle(gfxRect(0,0,0,0));
       ctx->Clip();
       AutoLayerManagerSetup setupLayerManager(this, ctx,
                                               BUFFER_NONE);
-      mAttachedWidgetListener->PaintWindow(this, region, 0);
-      mTabChild->NotifyPainted();
+      mAttachedWidgetListener->PaintWindow(this, region);
+      if (mTabChild) {
+        mTabChild->NotifyPainted();
+      }
     }
   }
 
@@ -606,13 +627,30 @@ float
 PuppetWidget::GetDPI()
 {
   if (mDPI < 0) {
-    NS_ABORT_IF_FALSE(mTabChild, "Need TabChild to get the DPI from!");
-    mTabChild->GetDPI(&mDPI);
+    if (mTabChild) {
+      mTabChild->GetDPI(&mDPI);
+    } else {
+      mDPI = 96.0;
+    }
   }
 
   return mDPI;
 }
 
+double
+PuppetWidget::GetDefaultScaleInternal()
+{
+  if (mDefaultScale < 0) {
+    if (mTabChild) {
+      mTabChild->GetDefaultScale(&mDefaultScale);
+    } else {
+      mDefaultScale = 1;
+    }
+  }
+
+  return mDefaultScale;
+}
+
 void*
 PuppetWidget::GetNativeData(uint32_t aDataType)
 {
@@ -620,7 +658,9 @@ PuppetWidget::GetNativeData(uint32_t aDataType)
   case NS_NATIVE_SHAREABLE_WINDOW: {
     NS_ABORT_IF_FALSE(mTabChild, "Need TabChild to get the nativeWindow from!");
     mozilla::WindowsHandle nativeData = 0;
-    mTabChild->SendGetWidgetNativeData(&nativeData);
+    if (mTabChild) {
+      mTabChild->SendGetWidgetNativeData(&nativeData);
+    }
     return (void*)nativeData;
   }
   case NS_NATIVE_WINDOW:
@@ -744,5 +784,12 @@ PuppetScreenManager::GetNumberOfScreens(uint32_t* aNumberOfScreens)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+PuppetScreenManager::GetSystemDefaultScale(float *aDefaultScale)
+{
+  *aDefaultScale = 1.0f;
+  return NS_OK;
+}
+
 }  // namespace widget
 }  // namespace mozilla
